
# q-expansion of newform 945.2.bj.h, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 945
weight = 2
poly_data = [3, -12, 19, -15, 10, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[6, 0, 1, 5, -1, 1], 3], [[0, -3, 2, -5, 1, -1], 3], [[6, -21, 19, -16, 5, -2], 3], [[-9, 33, -22, 19, -5, 2], 3]]

hecke_ring_character_values = [[596, [-1, 0, 0, 0, 0, 0]], [757, [1, 0, 0, 0, 0, 0]], [136, [0, 0, 0, 0, -1, 0]]]
aps_data = [[1, 0, 0, 1, 0, 0], [0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -1, 0], [0, -2, -1, 0, -1, 0], [0, -1, -1, 1, 0, -1], [1, 2, 1, 1, 2, -2], [-1, 2, 0, 2, -1, 0], [4, -1, -2, -2, 2, 1], [-1, 0, 0, 1, -1, 0], [1, 5, 3, 3, 1, -6], [-1, 2, 1, -2, 1, 1], [1, 2, 0, 1, -5, -4], [-5, 1, 0, -2, 1, 0], [0, 0, 4, 4, 0, 0], [-1, 2, 0, -1, 8, 0], [-4, 2, -1, -2, 4, -1], [-3, -3, -6, -3, -3, 6], [-9, -1, -2, -5, -3, 1], [-10, -5, -3, -5, -10, 3], [-6, 4, -2, -2, -4, 4], [0, -1, -2, 1, 0, -2], [-5, 7, 0, -5, 0, 3], [-8, -2, 0, 4, -2, 0], [4, 1, 0, 4, -5, -9], [-2, 8, 1, 1, 2, -2], [-3, 0, -3, 0, -3, 3], [-3, 4, 8, 5, 0, -4], [4, -5, -10, -6, 0, 5], [2, 1, 5, 1, 2, -5], [2, 0, -1, -1, 6, 2], [4, -1, 3, 5, -1, 0], [-5, 4, 0, -5, 10, 6], [5, -7, -4, 7, -5, -4], [-2, 2, 4, 4, -10, -8], [1, 0, 0, -5, 3, 0], [-3, -1, 2, -1, -3, -2], [-1, 0, -3, 0, 1, -3], [3, -3, 0, 3, -1, -3], [7, 4, 0, -8, 4, 0], [1, -5, 0, 1, 4, 3], [-1, 8, 2, -8, 1, 2], [-7, 5, -5, -5, 1, 10], [1, 7, 14, -3, 9, -7], [2, 1, 8, 1, 2, -8], [1, -3, -4, -4, 7, 8], [-1, -5, -1, 5, 1, -1], [8, -1, 1, 3, -1, 0], [9, 7, 2, 2, 21, -4], [-8, 4, 3, 4, -8, -3], [-1, -2, -4, -7, 1, 2], [21, 2, 4, 11, 7, -2], [9, -3, -1, -1, 17, 2], [14, -3, 0, 3, -14, 0], [-22, -1, -3, -1, -1, 0], [2, 5, 0, 2, -7, -9], [-9, 14, 2, -14, 9, 2], [-9, 0, -3, 0, -9, 3], [-5, -2, -4, -7, -1, 2], [1, 3, -6, 3, 1, 6], [-3, -11, -2, -2, -13, 4], [0, -2, -1, 2, 0, -1], [7, -2, -3, 1, -2, 0], [0, 6, 0, 0, 6, 0], [-22, -1, 3, -1, -22, -3], [21, 3, 6, 3, 12, -3], [21, -4, -8, -7, 10, 4], [2, 6, 0, 2, 14, -10], [2, -1, 4, 6, -1, 0], [14, -9, -2, 9, -14, -2], [-5, 1, -1, -1, -7, 2], [6, 6, 12, 6, 6, -12], [9, 1, 2, 3, 4, -1], [5, -13, 1, 13, -5, 1], [10, -11, 0, 10, -9, -9], [21, -6, 4, 16, -6, 0], [13, -20, 0, 13, -8, -6], [-5, 0, 4, 0, 5, 4], [0, -6, -12, -6, -3, 6], [-15, -5, -10, -9, -8, 5], [-9, 14, 7, -14, 9, 7], [-9, -3, -6, 0, -3, 0], [-14, 5, -3, -13, 5, 0], [-4, 8, 5, -8, 4, 5], [-13, 5, -5, -5, -11, 10], [5, -6, -12, -3, -2, 6], [4, 1, 2, 0, 3, -1], [3, -1, 0, 0, 5, 0], [3, 4, 0, 3, 0, -10], [10, -5, 3, 13, -5, 0], [-7, 9, 18, 0, 9, 0], [-1, 17, 0, -1, 14, -15], [-8, -11, -6, -11, -8, 6], [-14, 9, -1, 9, -14, 1], [19, -1, 6, 6, 25, -12], [-7, 6, 0, -7, 11, 8], [-14, 1, -12, -14, 1, 0], [3, -3, 0, 3, 21, -3], [-2, 13, 15, 13, -2, -15], [-8, -5, -10, -4, -7, 5], [2, 11, 0, 2, -14, -15], [-8, -1, -12, -10, -1, 0], [13, -19, -1, 19, -13, -1], [10, 10, 9, 10, 10, -9], [12, 1, 2, -6, 10, -1], [22, 3, 15, 3, 22, -15], [-1, -7, -8, 7, 1, -8], [-18, 3, 0, -6, 3, 0], [6, -21, 0, 6, -33, 9], [-1, -5, 6, 5, 1, 6], [11, -5, -1, -1, 19, 2], [14, 0, 0, 18, -2, 0], [-1, -14, -12, -14, -1, 12], [16, -1, 6, 6, 19, -12], [7, -11, 2, 11, -7, 2], [-10, 0, -6, -6, 0, 0], [17, -17, -3, 17, -17, -3], [-3, -7, -8, -8, 3, 16], [4, -14, -21, -14, 4, 21], [13, 7, 14, 27, 0, -7], [-6, 3, -4, -4, -1, 8], [12, -5, 2, 5, -12, 2], [-4, 8, -8, -24, 8, 0], [3, -21, 0, 3, -24, 15], [17, -6, -14, 6, -17, -14], [0, -1, -2, 22, -12, 1], [3, 0, -1, -1, 8, 2], [-13, 21, 0, -13, 17, 5], [-16, 20, 0, -16, 17, 12], [0, 1, 5, 5, -9, -10], [-12, 5, 10, 10, -6, -5], [2, 1, 9, 1, 2, -9], [18, -9, -1, -1, 29, 2], [14, -12, 0, 14, 8, -16], [26, -4, -2, 6, -4, 0], [0, -3, 0, 0, 9, 3], [4, -11, 2, 2, -7, -4], [-3, 6, 15, 6, -3, -15], [13, -22, -11, 22, -13, -11], [0, 9, 3, -15, 9, 0], [11, 3, -11, -3, -11, -11], [5, -15, 0, 0, -5, 0], [-23, 1, 2, -21, 0, -1], [-29, -9, -15, -9, -29, 15], [-11, -3, -1, -1, -23, 2], [23, -27, 6, 27, -23, 6], [-12, 9, -3, -21, 9, 0], [5, -13, 1, 1, -5, -2], [24, -3, 0, -3, 24, 0], [9, 9, 18, -3, 15, -9], [-13, -2, -4, -21, 2, 2], [-15, 12, 0, -15, 23, 18], [-12, 3, 12, 6, 3, 0], [-11, 7, -2, -16, 7, 0], [-4, -7, 0, -4, -10, 15], [-22, 7, 8, 7, -22, -8], [-1, -17, -8, -8, -3, 16], [13, -10, 0, 13, -17, -16], [9, -24, 0, 9, -39, 6], [10, 19, 11, 11, 17, -22], [-24, -9, -6, -9, -24, 6], [16, -5, -10, 6, 0, 5], [25, 11, 6, 6, 49, -12], [23, 3, -9, -15, 3, 0], [-14, 13, 0, -14, 4, 15], [-8, -6, 7, 6, 8, 7], [26, -4, -12, -4, 26, 12], [13, 18, 26, 18, 13, -26], [-8, -3, 0, 3, 8, 0]]
